/******************************************************************************
 * @file     main.c
 * @version  V1.00
 * $Revision: 3 $
 * $Date: 15/12/28 10:37a $
 * @brief    Nu-DIP-MINI 7 Segment LED
 *
 * @note
 * Copyright (C) 2015 Nuvoton Technology Corp. All rights reserved.
*****************************************************************************/
#include <stdio.h>
#include "mini58series.h"

#define PLL_CLOCK	50000000

void SYS_Init(void)
{
    /* Unlock protected registers */
    SYS_UnlockReg();

    /* Enable HIRC */
    CLK->PWRCTL = CLK_PWRCTL_HIRCEN_Msk;

    /* Waiting for clock ready */
    CLK_WaitClockReady(CLK_STATUS_HIRCSTB_Msk);

    /* Switch HCLK clock source to HIRC and HCLK srouce divide 1 */
    CLK_SetHCLK(CLK_CLKSEL0_HCLKSEL_HIRC, CLK_CLKDIV_HCLK(1));

	/* Set core clock as PLL_CLOCK from PLL */
	CLK_SetCoreClock(PLL_CLOCK);
	
	/* Select UART0 module clock source */
	CLK_SetModuleClock(UART0_MODULE, CLK_CLKSEL1_UARTSEL_PLL, CLK_CLKDIV_UART(1));
	
    /*---------------------------------------------------------------------------------------------------------*/
    /* Init I/O Multi-function                                                                                 */
    /*---------------------------------------------------------------------------------------------------------*/
	
	/* Set P5.0 pin for uart0 tx, P5.1 pin for uart0 rx */
	SYS->P5_MFP &= (SYS_MFP_P50_Msk | SYS_MFP_P51_Msk);
	SYS->P5_MFP |= (SYS_MFP_P50_UART0_TXD | SYS_MFP_P51_UART0_RXD);
	
	/* Set P04-P-7 multi-function pins for GPIO */
	SYS->P0_MFP &= ~(SYS_MFP_P07_Msk | SYS_MFP_P07_Msk | SYS_MFP_P07_Msk | SYS_MFP_P07_Msk);
	SYS->P0_MFP |= (SYS_MFP_P04_GPIO | SYS_MFP_P05_GPIO | SYS_MFP_P06_GPIO | SYS_MFP_P07_GPIO);
	
	/* Set P32,P34,P35 multi-function pins for GPIO */
	SYS->P2_MFP &= ~(SYS_MFP_P32_Msk | SYS_MFP_P34_Msk | SYS_MFP_P35_Msk);
	SYS->P2_MFP |= (SYS_MFP_P32_GPIO | SYS_MFP_P34_GPIO | SYS_MFP_P35_GPIO);
	
	/* Set P50 multi-function pins for GPIO */
	SYS->P5_MFP &= ~(SYS_MFP_P50_Msk);
	SYS->P5_MFP |= (SYS_MFP_P50_GPIO);

    /* To update the variable SystemCoreClock */
    SystemCoreClockUpdate();

    /* Lock protected registers */
    SYS_LockReg();
}

void UART0_Init(void)
{
	/* Reset UART0 */
	SYS_ResetModule(UART0_RST);
	
	/* Configure UART0 and set baudrate to 115200 */
	UART_Open(UART0, 115200);
}

void delay_loop(void)
{
	__IO uint32_t i;
	
	for(i = 0; i < 65000; i++);
	for(i = 0; i < 65000; i++);
	for(i = 0; i < 65000; i++);
}


/**************************************************
 * 7-seg	G	F	A	B	E	D	C	dot 
 *			P04	P05	P06	P07	P32	P34	P35	P50
 *	9		0	0	0	0	1	1	0	1
 *	8		0	0	0	0	0	0	0	1
 *	7		1	1	0	0	1	1	0	1
 *	6		0	0	0	1	0	0	0	1
 *	5		0	0	0	1	1	0	0	1
 *	4		0	0	1	0	1	1	0	1
 *	3		0	1	0	0	1	0	0	1
 *	2		0	1	0	0	0	0	1	1
 *	1		1	1	1	0	1	1	0	1
 *	0		1	0	0	0	0	0	0	1
**************************************************/
void seven_seg_show(uint8_t number)
{
	switch(number)
	{
		case 9:
			GPIO_SET_OUT_DATA(P0,0x00);
			GPIO_SET_OUT_DATA(P3,0x14);
			break;
		case 8:
			GPIO_SET_OUT_DATA(P0,0x00);
			GPIO_SET_OUT_DATA(P3,0x00);
			break;
		case 7:
			GPIO_SET_OUT_DATA(P0,0x30);
			GPIO_SET_OUT_DATA(P3,0x14);
			break;
		case 6:
			GPIO_SET_OUT_DATA(P0,0x80);
			GPIO_SET_OUT_DATA(P3,0x00);
			break;
		case 5:
			GPIO_SET_OUT_DATA(P0,0x80);
			GPIO_SET_OUT_DATA(P3,0x04);
			break;
		case 4:
			GPIO_SET_OUT_DATA(P0,0x40);
			GPIO_SET_OUT_DATA(P3,0x14);
			break;
		case 3:
			GPIO_SET_OUT_DATA(P0,0x20);
			GPIO_SET_OUT_DATA(P3,0x04);
			break;
		case 2:
			GPIO_SET_OUT_DATA(P0,0x20);
			GPIO_SET_OUT_DATA(P3,0x20);
			break;
		case 1:
			GPIO_SET_OUT_DATA(P0,0x70);
			GPIO_SET_OUT_DATA(P3,0x14);
			break;
		case 0:
			GPIO_SET_OUT_DATA(P0,0x10);
			GPIO_SET_OUT_DATA(P3,0x00);
			break;
	}
}

int main(void)
{
	uint8_t i;
	
	/* Init System, IP clock and multi-function I/O */
	SYS_Init();

	/* Init Uart0 for printf */
	UART0_Init();
	
    /*set P04,P05,P06,P07 to output mode */
    GPIO_SetMode(P0, (BIT4 | BIT5 | BIT6 | BIT7), GPIO_MODE_OUTPUT);
	
    /*set P32,P34,P35 to output mode */
    GPIO_SetMode(P3, (BIT2 | BIT4 | BIT5), GPIO_MODE_OUTPUT);
	
	/*set P50 to output mode */
    GPIO_SetMode(P5, BIT0, GPIO_MODE_OUTPUT);
	
	printf("\n\n CPU @ %d Hz \n", SystemCoreClock);
	printf("+------------------------------------------------+\n");
	printf("+----------- 7 Segment LED Sample code ----------+\n");
	printf("+------------------------------------------------+\n");
	
	while(1)
	{
		for( i = 0; i < 10 ; i++)
		{
			seven_seg_show(i);
			delay_loop();
			delay_loop();
			delay_loop();
			delay_loop();
			delay_loop();
		}
	}
}	


