/******************************************************************************
 * @file     main.c
 * @version  V1.00
 * $Revision: 3 $
 * $Date: 15/12/28 10:37a $
 * @brief    Nu-DIP-MINI Dancing LED
 *
 * @note
 * Copyright (C) 2015 Nuvoton Technology Corp. All rights reserved.
*****************************************************************************/
#include <stdio.h>
#include "mini58series.h"

#define PLL_CLOCK	50000000

void SYS_Init(void)
{
    /* Unlock protected registers */
    SYS_UnlockReg();

    /* Enable HIRC */
    CLK->PWRCTL = CLK_PWRCTL_HIRCEN_Msk;

    /* Waiting for clock ready */
    CLK_WaitClockReady(CLK_STATUS_HIRCSTB_Msk);

    /* Switch HCLK clock source to HIRC and HCLK srouce divide 1 */
    CLK_SetHCLK(CLK_CLKSEL0_HCLKSEL_HIRC, CLK_CLKDIV_HCLK(1));

	/* Set core clock as PLL_CLOCK from PLL */
	CLK_SetCoreClock(PLL_CLOCK);
	
	/* Select UART0 module clock source */
	CLK_SetModuleClock(UART0_MODULE, CLK_CLKSEL1_UARTSEL_PLL, CLK_CLKDIV_UART(1));
	
    /*---------------------------------------------------------------------------------------------------------*/
    /* Init I/O Multi-function                                                                                 */
    /*---------------------------------------------------------------------------------------------------------*/
	/* Set P5.0 pin for uart0 tx, P5.1 pin for uart0 rx */
	SYS->P5_MFP &= (SYS_MFP_P50_Msk | SYS_MFP_P51_Msk);
	SYS->P5_MFP |= (SYS_MFP_P50_UART0_TXD | SYS_MFP_P51_UART0_RXD);
	
	/* Set P0 multi-function pins for GPIO */
	SYS->P0_MFP &= ~(SYS_MFP_P04_Msk | SYS_MFP_P05_Msk | SYS_MFP_P06_Msk | SYS_MFP_P07_Msk);
	SYS->P0_MFP |= (SYS_MFP_P04_GPIO | SYS_MFP_P05_GPIO | SYS_MFP_P06_GPIO | SYS_MFP_P07_GPIO);

    /* To update the variable SystemCoreClock */
    SystemCoreClockUpdate();

    /* Lock protected registers */
    SYS_LockReg();
}

void UART0_Init(void)
{
	/* Reset UART0 */
	SYS_ResetModule(UART0_RST);
	
	/* Configure UART0 and set baudrate to 115200 */
	UART_Open(UART0, 115200);
}

void delay_loop(void)
{
	__IO uint32_t i;
	for(i = 0; i < 65000; i++);
	for(i = 0; i < 65000; i++);
	for(i = 0; i < 65000; i++);
}

int main(void)
{
	uint8_t i,p0_data;
	
	/* Init System, IP clock and multi-function I/O */
	SYS_Init();
	
	/* Init Uart0 for printf */
	UART0_Init();
	
    /*set P4.5, P4.6, P4.7, P4.8 to output mode */
    GPIO_SetMode(P0, BIT4 | BIT5 | BIT6 | BIT7, GPIO_MODE_OUTPUT);

	printf("\n\n CPU @ %d Hz \n", SystemCoreClock);
	printf("+------------------------------------------------+\n");
	printf("+------------ Dancing LED Sample code -----------+\n");
	printf("+------------------------------------------------+\n");
	
	
	while(1)
	{
		p0_data = 0x10;	// init LED bar data
		for( i = 0; i < 4; i++)
		{
			GPIO_SET_OUT_DATA(P0, ~(p0_data << i));	// shift to left
			delay_loop();
		}
		
		p0_data = 0x80;	// init LED bar data
		for( i = 0; i < 4; i++)
		{
			GPIO_SET_OUT_DATA(P0, ~(p0_data >> i));	// shift to right
			delay_loop();
		}
	}
}	


