/******************************************************************************
 * @file     main.c
 * @version  V1.00
 * $Revision: 3 $
 * $Date: 15/12/28 10:37a $
 * @brief    Nu-DIP-MINI PWM Breathing Light
 *
 * @note
 * Copyright (C) 2015 Nuvoton Technology Corp. All rights reserved.
*****************************************************************************/
#include <stdio.h>
#include <math.h>
#include "mini58series.h"

#define PLL_CLOCK	50000000
#define PI			3.1415f

void SYS_Init(void)
{
    /* Unlock protected registers */
    SYS_UnlockReg();

    /* Enable HIRC */
    CLK->PWRCTL = CLK_PWRCTL_HIRCEN_Msk;

    /* Waiting for clock ready */
    CLK_WaitClockReady(CLK_STATUS_HIRCSTB_Msk);

    /* Switch HCLK clock source to HIRC and HCLK srouce divide 1 */
    CLK_SetHCLK(CLK_CLKSEL0_HCLKSEL_HIRC, CLK_CLKDIV_HCLK(1));

	/* Set core clock as PLL_CLOCK from PLL */
	CLK_SetCoreClock(PLL_CLOCK);
	
	/* Enable UART0 module clock */
	CLK_EnableModuleClock(UART0_MODULE);
	
	/* Select UART0 module clock source */
	CLK_SetModuleClock(UART0_MODULE, CLK_CLKSEL1_UARTSEL_PLL, CLK_CLKDIV_UART(1));
	
	/* Enable PWM channel 0,1 */
	CLK_EnableModuleClock(PWMCH45_MODULE);
	/* Set PWM channel 0,1 clock source */
	CLK_SetModuleClock(PWMCH45_MODULE, CLK_CLKSEL1_PWMCH01SEL_HCLK, NULL);
	
    /*---------------------------------------------------------------------------------------------------------*/
    /* Init I/O Multi-function                                                                                 */
    /*---------------------------------------------------------------------------------------------------------*/
	/* Set P5.0 pin for uart0 tx, P5.1 pin for uart0 rx */
	SYS->P5_MFP &= (SYS_MFP_P50_Msk | SYS_MFP_P51_Msk);
	SYS->P5_MFP |= (SYS_MFP_P50_UART0_TXD | SYS_MFP_P51_UART0_RXD);

	/* Set P0.4 pin for PWM channel 5 */
	SYS->P0_MFP &= SYS_MFP_P04_Msk;
	SYS->P0_MFP |= SYS_MFP_P04_PWM0_CH5;

    /* To update the variable SystemCoreClock */
    SystemCoreClockUpdate();

    /* Lock protected registers */
    SYS_LockReg();
}

void UART0_Init(void)
{
	/* Reset UART0 */
	SYS_ResetModule(UART0_RST);
	
	/* Configure UART0 and set baudrate to 115200 */
	UART_Open(UART0, 115200);
}

void PWM_Init(void)
{
	/* Reset PWM module */
	SYS_ResetModule(PWM0_RST);
	
	/* Enable PWM channel 5 output */
	PWM_EnableOutput(PWM, BIT5);
	
	/* Config PWM channel 5 to 500Hz */
	PWM_ConfigOutputChannel(PWM, PWM_CH5, 500, 10);
	
	/* Start PWM channel 5 */
	PWM_Start(PWM,BIT5);
}

void delay_loop(void)
{
	__IO uint32_t i;
	
	for(i = 0; i < 65000; i++);
	for(i = 0; i < 65000; i++);
	for(i = 0; i < 65000; i++);
}

int main(void)
{
	uint32_t brightness;
	double index = 0;
	
	/* Init System, IP clock and multi-function I/O */
	SYS_Init();

	/* Init Uart0 for printf */
	UART0_Init();
	
	/* Init PWM function */
	PWM_Init();
	
	printf("\n\n CPU @ %d Hz \n", SystemCoreClock);
	printf("+------------------------------------------------+\n");
	printf("+----- PWM Breathing Light ADC Sample code ------+\n");
	printf("+------------------------------------------------+\n");

	while(1)
	{
		brightness = floor(100 * sin(index));
		
		if (brightness == 0)
			brightness = 1;
		
		PWM_ConfigOutputChannel(PWM, PWM_CH5, 500, brightness);
		
		CLK_SysTickDelay(10000);
		
		index += PI/200;
		
		if (index > (2 * PI))
			index = 0;
	}
}	


