/******************************************************************************
 * @file     main.c
 * @version  V1.00
 * $Revision: 3 $
 * $Date: 15/12/28 10:37a $
 * @brief    Nu-DIP-MINI PhotoResistor ADC
 *
 * @note
 * Copyright (C) 2015 Nuvoton Technology Corp. All rights reserved.
*****************************************************************************/
#include <stdio.h>
#include "mini58series.h"

#define PLL_CLOCK	50000000

void ADC_IRQHandler(void)
{
    uint32_t u32Flag;

    /* Get ADC interrupt flag */
    u32Flag = ADC_GET_INT_FLAG(ADC, ADC_ADIF_INT);

    /* Get ADC convert result */
    printf("Light data --> %0000d \n", (uint32_t)ADC_GET_CONVERSION_DATA(ADC, 2));

	/* Clear ADC interrupt flag */
    ADC_CLR_INT_FLAG(ADC, u32Flag);
}

void SYS_Init(void)
{
    /* Unlock protected registers */
    SYS_UnlockReg();

    /* Enable HIRC */
    CLK->PWRCTL = CLK_PWRCTL_HIRCEN_Msk;

    /* Waiting for clock ready */
    CLK_WaitClockReady(CLK_STATUS_HIRCSTB_Msk);

    /* Switch HCLK clock source to HIRC and HCLK srouce divide 1 */
    CLK_SetHCLK(CLK_CLKSEL0_HCLKSEL_HIRC, CLK_CLKDIV_HCLK(1));

	/* Set core clock as PLL_CLOCK from PLL */
	CLK_SetCoreClock(PLL_CLOCK);
	
	/* Enable UART0 module clock */
	CLK_EnableModuleClock(UART0_MODULE);
	
	/* Select UART0 module clock source */
	CLK_SetModuleClock(UART0_MODULE, CLK_CLKSEL1_UARTSEL_PLL, CLK_CLKDIV_UART(1));
	
	/* Enable ADC moduel clock */
	CLK_EnableModuleClock(ADC_MODULE);
	
	/* Select ADC module clock source */
	CLK_SetModuleClock(ADC_MODULE,  CLK_CLKSEL1_ADCSEL_HIRC, CLK_CLKDIV_ADC(1));
	
    /*---------------------------------------------------------------------------------------------------------*/
    /* Init I/O Multi-function                                                                                 */
    /*---------------------------------------------------------------------------------------------------------*/

	/* Set P5.0 pin for uart0 tx, P5.1 pin for uart0 rx */
	SYS->P5_MFP &= (SYS_MFP_P50_Msk | SYS_MFP_P51_Msk);
	SYS->P5_MFP |= (SYS_MFP_P50_UART0_TXD | SYS_MFP_P51_UART0_RXD);
	
	/* Set P1.2 pin for adc channel 2 */
	SYS->P1_MFP &= (SYS_MFP_P12_Msk);
	SYS->P1_MFP |= (SYS_MFP_P12_ADC_CH2);


    /* To update the variable SystemCoreClock */
    SystemCoreClockUpdate();

    /* Lock protected registers */
    SYS_LockReg();
}

void UART0_Init(void)
{
	/* Reset UART0 */
	SYS_ResetModule(UART0_RST);
	
	/* Configure UART0 and set baudrate to 115200 */
	UART_Open(UART0, 115200);
}

void ADC_Init(void)
{
	/* Enable ADC channel 2 */
	ADC_Open(ADC, NULL, NULL, 0x01 << 2);

    /* Enable ADC convert complete interrupt */
    ADC_EnableInt(ADC, ADC_ADIF_INT);
    NVIC_EnableIRQ(ADC_IRQn);
	
	/* Power on ADC */
	ADC_POWER_ON(ADC);
}

void delay_loop(void)
{
	__IO uint32_t i;
	
	for(i = 0; i < 65000; i++);
	for(i = 0; i < 65000; i++);
	for(i = 0; i < 65000; i++);
}

int main(void)
{
	/* Init System, IP clock and multi-function I/O */
	SYS_Init();

	/* Init Uart0 for printf */
	UART0_Init();
	
	/* Init ADC */
	ADC_Init();
	
	printf("\n\n CPU @ %d Hz \n", SystemCoreClock);
	printf("+------------------------------------------------+\n");
	printf("+--------- PhotoResistor ADC Sample code --------+\n");
	printf("+------------------------------------------------+\n");
	
	while(1)
	{
        // Trigger ADC conversion if it is idle
        if(!ADC_IS_BUSY(ADC)) 
		{
            ADC_START_CONV(ADC);
			delay_loop();
        }
	}
}	


